/*  Note: You are free to use whatever license you want.
    Eventually you will be able to edit it within Glade. */

/*  gHostEdit
 *  Copyright (C) <YEAR> <AUTHORS>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "ghost-edit-sig.h"
#include "ghost-edit.h"

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("pixmaps", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_main ()
{
  GtkWidget *main;
  GtkWidget *vbox2;
  GtkWidget *packer3;
  GtkWidget *handlebox2;
  GtkWidget *menubar;
  GtkWidget *Host_Table;
  GtkWidget *Help;
  GtkWidget *hbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *clist_HostTable;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *vbox3;
  GtkWidget *label4;
  GtkWidget *packer13;
  GtkWidget *hseparator4;
  GtkWidget *label5;
  GtkWidget *entry_IP;
  GtkWidget *label6;
  GtkWidget *entry_Hostname;
  GtkWidget *label7;
  GtkWidget *entry_Aliases;
  GtkWidget *hseparator5;
  GtkWidget *packer14;
  GtkWidget *label8;
  GtkWidget *text_Comments;
  GtkWidget *packer5;
  GtkWidget *hbuttonbox2;
  GtkWidget *btn_Add;
  GtkWidget *btn_Edit;
  GtkWidget *btn_Remove;
  GtkWidget *btn_Save;
  GtkWidget *btn_Close;
  GtkWidget *arrow1;
  GtkWidget *packer15;
  GtkWidget *statusbar_main;

  main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (main), "main", main);
  gtk_widget_set_usize (main, 589, -2);
  gtk_signal_connect (GTK_OBJECT (main), "focus_in_event",
                      GTK_SIGNAL_FUNC (on_main_focus_in_event),
                      GTK_OBJECT(main));
  gtk_signal_connect (GTK_OBJECT (main), "realize",
                      GTK_SIGNAL_FUNC (on_main_realize),
                      GTK_OBJECT(main));
  gtk_window_set_title (GTK_WINDOW (main), "ghost-edit");
  gtk_window_set_policy (GTK_WINDOW (main), TRUE, TRUE, TRUE);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (main), vbox2);

  packer3 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (main), "packer3", packer3);
  gtk_widget_show (packer3);
  gtk_box_pack_start (GTK_BOX (vbox2), packer3, FALSE, TRUE, 0);

  handlebox2 = gtk_handle_box_new ();
  gtk_object_set_data (GTK_OBJECT (main), "handlebox2", handlebox2);
  gtk_widget_show (handlebox2);
  gtk_packer_add (GTK_PACKER (packer3), handlebox2, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, GTK_FILL_X,
                  0, 0, 0, 0, 0);
  gtk_widget_set_usize (handlebox2, -2, 25);

  menubar = gtk_menu_bar_new ();
  gtk_object_set_data (GTK_OBJECT (main), "menubar", menubar);
  gtk_widget_show (menubar);
  gtk_container_add (GTK_CONTAINER (handlebox2), menubar);

  Host_Table = gtk_menu_item_new_with_label ("Host Table");
  gtk_object_set_data (GTK_OBJECT (main), "Host_Table", Host_Table);
  gtk_widget_show (Host_Table);
  gtk_container_add (GTK_CONTAINER (menubar), Host_Table);
  gtk_signal_connect (GTK_OBJECT (Host_Table), "activate",
                      GTK_SIGNAL_FUNC (on_Host_Table_activate),
                      NULL);

  Help = gtk_menu_item_new_with_label ("Help");
  gtk_object_set_data (GTK_OBJECT (main), "Help", Help);
  gtk_widget_show (Help);
  gtk_container_add (GTK_CONTAINER (menubar), Help);
  gtk_signal_connect (GTK_OBJECT (Help), "activate",
                      GTK_SIGNAL_FUNC (on_Help_activate),
                      NULL);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main), "scrolledwindow1", scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox1), scrolledwindow1, TRUE, TRUE, 10);
  gtk_widget_set_usize (scrolledwindow1, 330, 319);

  clist_HostTable = gtk_clist_new (3);
  gtk_object_set_data (GTK_OBJECT (main), "clist_HostTable", clist_HostTable);
  gtk_widget_show (clist_HostTable);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), clist_HostTable);
  gtk_signal_connect (GTK_OBJECT (clist_HostTable), "select_row",
                      GTK_SIGNAL_FUNC (on_clist_HostTable_select_row),
                      GTK_OBJECT(main));
  gtk_clist_set_column_width (GTK_CLIST (clist_HostTable), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist_HostTable), 1, 203);
  gtk_clist_set_column_width (GTK_CLIST (clist_HostTable), 2, 363);
  gtk_clist_column_titles_show (GTK_CLIST (clist_HostTable));

  label1 = gtk_label_new ("IP Address");
  gtk_object_set_data (GTK_OBJECT (main), "label1", label1);
  gtk_widget_show (label1);
  gtk_clist_set_column_widget (GTK_CLIST (clist_HostTable), 0, label1);

  label2 = gtk_label_new ("Hostname");
  gtk_object_set_data (GTK_OBJECT (main), "label2", label2);
  gtk_widget_show (label2);
  gtk_clist_set_column_widget (GTK_CLIST (clist_HostTable), 1, label2);

  label3 = gtk_label_new ("Alias");
  gtk_object_set_data (GTK_OBJECT (main), "label3", label3);
  gtk_widget_show (label3);
  gtk_clist_set_column_widget (GTK_CLIST (clist_HostTable), 2, label3);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main), "vbox3", vbox3);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox3, TRUE, TRUE, 10);
  gtk_widget_set_usize (vbox3, 120, -2);

  label4 = gtk_label_new ("Enter/Edit Host Information");
  gtk_object_set_data (GTK_OBJECT (main), "label4", label4);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox3), label4, FALSE, FALSE, 5);

  packer13 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (main), "packer13", packer13);
  gtk_widget_show (packer13);
  gtk_box_pack_start (GTK_BOX (vbox3), packer13, TRUE, TRUE, 0);

  hseparator4 = gtk_hseparator_new ();
  gtk_object_set_data (GTK_OBJECT (main), "hseparator4", hseparator4);
  gtk_widget_show (hseparator4);
  gtk_packer_add (GTK_PACKER (packer13), hseparator4, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, GTK_FILL_X,
                  0, 0, 7, 0, 0);

  label5 = gtk_label_new ("IP Address");
  gtk_object_set_data (GTK_OBJECT (main), "label5", label5);
  gtk_widget_show (label5);
  gtk_packer_add (GTK_PACKER (packer13), label5, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 10, 0, 0, 0);

  entry_IP = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (main), "entry_IP", entry_IP);
  gtk_widget_show (entry_IP);
  gtk_packer_add (GTK_PACKER (packer13), entry_IP, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (entry_IP), "insert_text",
                      GTK_SIGNAL_FUNC (on_entry_IP_insert_text),
                      GTK_OBJECT(main));
  gtk_signal_connect (GTK_OBJECT (entry_IP), "enter_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_IP_enter_notify_event),
                      "Enter the host's IP address here...");
  gtk_signal_connect (GTK_OBJECT (entry_IP), "leave_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_IP_leave_notify_event),
                      "Welcome to gHost-Edit!");

  label6 = gtk_label_new ("Hostname");
  gtk_object_set_data (GTK_OBJECT (main), "label6", label6);
  gtk_widget_show (label6);
  gtk_packer_add (GTK_PACKER (packer13), label6, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 10, 0, 0, 0);

  entry_Hostname = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (main), "entry_Hostname", entry_Hostname);
  gtk_widget_show (entry_Hostname);
  gtk_packer_add (GTK_PACKER (packer13), entry_Hostname, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (entry_Hostname), "insert_text",
                      GTK_SIGNAL_FUNC (on_entry_Hostname_insert_text),
                      GTK_OBJECT(main));
  gtk_signal_connect (GTK_OBJECT (entry_Hostname), "enter_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_Hostname_enter_notify_event),
                      "Enter the host's valid Hostname here...");
  gtk_signal_connect (GTK_OBJECT (entry_Hostname), "leave_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_Hostname_leave_notify_event),
                      "Welcome to gHost-Edit!");

  label7 = gtk_label_new ("Alias");
  gtk_object_set_data (GTK_OBJECT (main), "label7", label7);
  gtk_widget_show (label7);
  gtk_packer_add (GTK_PACKER (packer13), label7, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 10, 0, 0, 0);

  entry_Aliases = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (main), "entry_Aliases", entry_Aliases);
  gtk_widget_show (entry_Aliases);
  gtk_packer_add (GTK_PACKER (packer13), entry_Aliases, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (entry_Aliases), "insert_text",
                      GTK_SIGNAL_FUNC (on_entry_Aliases_insert_text),
                      GTK_OBJECT(main));
  gtk_signal_connect (GTK_OBJECT (entry_Aliases), "enter_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_Aliases_enter_notify_event),
                      "Enter any aliases or nicknames for your host here...");
  gtk_signal_connect (GTK_OBJECT (entry_Aliases), "leave_notify_event",
                      GTK_SIGNAL_FUNC (on_entry_Aliases_leave_notify_event),
                      "Welcome to gHost-Edit!");

  hseparator5 = gtk_hseparator_new ();
  gtk_object_set_data (GTK_OBJECT (main), "hseparator5", hseparator5);
  gtk_widget_show (hseparator5);
  gtk_packer_add (GTK_PACKER (packer13), hseparator5, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, GTK_FILL_X,
                  0, 0, 19, 0, 0);

  packer14 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (main), "packer14", packer14);
  gtk_widget_show (packer14);
  gtk_box_pack_start (GTK_BOX (vbox3), packer14, TRUE, TRUE, 0);

  label8 = gtk_label_new ("Comments");
  gtk_object_set_data (GTK_OBJECT (main), "label8", label8);
  gtk_widget_show (label8);
  gtk_packer_add (GTK_PACKER (packer14), label8, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);

  text_Comments = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main), "text_Comments", text_Comments);
  gtk_widget_show (text_Comments);
  gtk_packer_add (GTK_PACKER (packer14), text_Comments, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (text_Comments), "insert_text",
                      GTK_SIGNAL_FUNC (on_text_Comments_insert_text),
                      GTK_OBJECT(main));
  gtk_widget_realize (text_Comments);
  gtk_text_insert (GTK_TEXT (text_Comments), NULL, NULL, NULL,
                   "Coming soon...", 14);

  packer5 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (main), "packer5", packer5);
  gtk_widget_show (packer5);
  gtk_box_pack_start (GTK_BOX (vbox2), packer5, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (packer5), 5);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (main), "hbuttonbox2", hbuttonbox2);
  gtk_widget_show (hbuttonbox2);
  gtk_packer_add (GTK_PACKER (packer5), hbuttonbox2, GTK_SIDE_RIGHT,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox2), GTK_BUTTONBOX_EDGE);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox2), 15);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox2), 85, 20);

  btn_Add = gtk_button_new_with_label ("Add");
  gtk_object_set_data (GTK_OBJECT (main), "btn_Add", btn_Add);
  gtk_widget_show (btn_Add);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_Add);
  gtk_widget_set_sensitive (btn_Add, FALSE);
  gtk_signal_connect (GTK_OBJECT (btn_Add), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_Add_clicked),
                      GTK_OBJECT(main));

  btn_Edit = gtk_button_new_with_label ("Update");
  gtk_object_set_data (GTK_OBJECT (main), "btn_Edit", btn_Edit);
  gtk_widget_show (btn_Edit);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_Edit);
  gtk_widget_set_sensitive (btn_Edit, FALSE);
  gtk_signal_connect (GTK_OBJECT (btn_Edit), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_Edit_clicked),
                      GTK_OBJECT(main));

  btn_Remove = gtk_button_new_with_label ("Remove");
  gtk_object_set_data (GTK_OBJECT (main), "btn_Remove", btn_Remove);
  gtk_widget_show (btn_Remove);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_Remove);
  gtk_signal_connect (GTK_OBJECT (btn_Remove), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_Remove_clicked),
                      GTK_OBJECT(main));

  btn_Save = gtk_button_new_with_label ("Save");
  gtk_object_set_data (GTK_OBJECT (main), "btn_Save", btn_Save);
  gtk_widget_show (btn_Save);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_Save);
  gtk_signal_connect (GTK_OBJECT (btn_Save), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_Save_clicked),
                      GTK_OBJECT(main));

  btn_Close = gtk_button_new_with_label ("Close");
  gtk_object_set_data (GTK_OBJECT (main), "btn_Close", btn_Close);
  gtk_widget_show (btn_Close);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), btn_Close);
  gtk_signal_connect (GTK_OBJECT (btn_Close), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_Close_clicked),
                      NULL);

  arrow1 = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
  gtk_object_set_data (GTK_OBJECT (main), "arrow1", arrow1);
  gtk_packer_add (GTK_PACKER (packer5), arrow1, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_widget_set_usize (arrow1, 9, 9);
  gtk_widget_set_sensitive (arrow1, FALSE);

  packer15 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (main), "packer15", packer15);
  gtk_widget_show (packer15);
  gtk_box_pack_start (GTK_BOX (vbox2), packer15, TRUE, TRUE, 0);

  statusbar_main = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (main), "statusbar_main", statusbar_main);
  gtk_widget_show (statusbar_main);
  gtk_packer_add (GTK_PACKER (packer15), statusbar_main, GTK_SIDE_TOP,
                  GTK_ANCHOR_EAST, GTK_FILL_X,
                  0, 0, 0, 0, 0);

  return main;
}

GtkWidget*
create_popup_Main1 ()
{
  GtkWidget *popup_Main1;
  GtkWidget *popup_Main;

  popup_Main1 = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (popup_Main1), "popup_Main1", popup_Main1);
  gtk_signal_connect (GTK_OBJECT (popup_Main1), "selection_done",
                      GTK_SIGNAL_FUNC (on_popup_Main_selection_done),
                      NULL);

  popup_Main = gtk_menu_item_new_with_label ("popup_Main");
  gtk_object_set_data (GTK_OBJECT (popup_Main1), "popup_Main", popup_Main);
  gtk_widget_show (popup_Main);
  gtk_container_add (GTK_CONTAINER (popup_Main1), popup_Main);
  gtk_signal_connect (GTK_OBJECT (popup_Main), "activate",
                      GTK_SIGNAL_FUNC (on_popup_Main_activate),
                      NULL);

  return popup_Main1;
}

GtkWidget*
create_fileselection1 ()
{
  GtkWidget *fileselection1;
  GtkWidget *ok_button1;
  GtkWidget *cancel_button1;

  fileselection1 = gtk_file_selection_new ("Select Host Table");
  gtk_object_set_data (GTK_OBJECT (fileselection1), "fileselection1", fileselection1);
  gtk_container_border_width (GTK_CONTAINER (fileselection1), 10);
  GTK_WINDOW (fileselection1)->type = GTK_WINDOW_DIALOG;

  ok_button1 = GTK_FILE_SELECTION (fileselection1)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "ok_button1", ok_button1);
  gtk_widget_show (ok_button1);
  GTK_WIDGET_SET_FLAGS (ok_button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (ok_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_ok_button1_clicked),
                      GTK_FILE_SELECTION(fileselection1));

  cancel_button1 = GTK_FILE_SELECTION (fileselection1)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "cancel_button1", cancel_button1);
  gtk_widget_show (cancel_button1);
  GTK_WIDGET_SET_FLAGS (cancel_button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (cancel_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_cancel_button1_clicked),
                      GTK_FILE_SELECTION(fileselection1));

  return fileselection1;
}

GtkWidget*
create_saveComplete ()
{
  GtkWidget *saveComplete;
  GtkWidget *vbox4;
  GtkWidget *packer17;
  GtkWidget *lbl_SaveCompleted;
  GtkWidget *savedHostFile;
  GtkWidget *packer18;
  GtkWidget *savedOK;

  saveComplete = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (saveComplete), "saveComplete", saveComplete);
  gtk_window_set_title (GTK_WINDOW (saveComplete), "Data Saved");
  gtk_window_set_policy (GTK_WINDOW (saveComplete), FALSE, FALSE, FALSE);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (saveComplete), "vbox4", vbox4);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (saveComplete), vbox4);

  packer17 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (saveComplete), "packer17", packer17);
  gtk_widget_show (packer17);
  gtk_box_pack_start (GTK_BOX (vbox4), packer17, TRUE, TRUE, 0);

  lbl_SaveCompleted = gtk_label_new ("Saved Host data to file:");
  gtk_object_set_data (GTK_OBJECT (saveComplete), "lbl_SaveCompleted", lbl_SaveCompleted);
  gtk_widget_show (lbl_SaveCompleted);
  gtk_packer_add (GTK_PACKER (packer17), lbl_SaveCompleted, GTK_SIDE_BOTTOM,
                  GTK_ANCHOR_SOUTH, 0,
                  0, 0, 0, 0, 0);
  gtk_misc_set_padding (GTK_MISC (lbl_SaveCompleted), 30, 10);

  savedHostFile = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (saveComplete), "savedHostFile", savedHostFile);
  gtk_widget_show (savedHostFile);
  gtk_box_pack_start (GTK_BOX (vbox4), savedHostFile, TRUE, TRUE, 0);
  gtk_misc_set_padding (GTK_MISC (savedHostFile), 15, 5);

  packer18 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (saveComplete), "packer18", packer18);
  gtk_widget_show (packer18);
  gtk_box_pack_start (GTK_BOX (vbox4), packer18, TRUE, FALSE, 0);

  savedOK = gtk_button_new_with_label ("OK");
  gtk_object_set_data (GTK_OBJECT (saveComplete), "savedOK", savedOK);
  gtk_widget_show (savedOK);
  gtk_packer_add (GTK_PACKER (packer18), savedOK, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  15, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (savedOK), "clicked",
                      GTK_SIGNAL_FUNC (on_savedOK_clicked),
                      GTK_WIDGET(saveComplete));

  return saveComplete;
}

GtkWidget*
create_fileError ()
{
  GtkWidget *fileError;
  GtkWidget *packer19;
  GtkWidget *noWayJose;
  GtkWidget *ok;

  fileError = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (fileError), "fileError", fileError);
  gtk_window_set_title (GTK_WINDOW (fileError), "Ooops!");
  gtk_window_position (GTK_WINDOW (fileError), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (fileError), FALSE, FALSE, FALSE);

  packer19 = gtk_packer_new ();
  gtk_object_set_data (GTK_OBJECT (fileError), "packer19", packer19);
  gtk_widget_show (packer19);
  gtk_container_add (GTK_CONTAINER (fileError), packer19);

  noWayJose = gtk_label_new ("Sorry!  I couldn't find that file...");
  gtk_object_set_data (GTK_OBJECT (fileError), "noWayJose", noWayJose);
  gtk_widget_show (noWayJose);
  gtk_packer_add (GTK_PACKER (packer19), noWayJose, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_misc_set_padding (GTK_MISC (noWayJose), 0, 23);

  ok = gtk_button_new_with_label ("Well, Ok...");
  gtk_object_set_data (GTK_OBJECT (fileError), "ok", ok);
  gtk_widget_show (ok);
  gtk_packer_add (GTK_PACKER (packer19), ok, GTK_SIDE_TOP,
                  GTK_ANCHOR_CENTER, 0,
                  0, 0, 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
                      GTK_SIGNAL_FUNC (on_ok_clicked),
                      GTK_OBJECT(fileError));

  return fileError;
}

